<?php

/********************************************
* LiteSpeed AP2ZConf project
* @Author:   LiteSpeed Technologies, Inc. (http://www.litespeedtech.com)
* @Copyright: (c) 2017
*********************************************/

require_once('MigrUtil.php');
require_once('ParsedConf.php');

class FileLoader
{

	private static $SupportedModules = array(
		'expires',
		'alias',
		'rewrite',
		'access',
		'auth',
		'ssl',
		'zconf'
	);

	private static $AllowDupBlock = array(
		'VirtualHost'
	);

	var $_apacheServerRoot;
	var $_debug = 0;

	/**
	 * Load and parse the Apache configurations.
	 *
	 * @param string $apache_conf_path Absolute path to the main apache configurations file.
	 */
	function &load($apache_conf_path)
	{
		$lines = file($apache_conf_path);
		if ($lines == false)
		{
			echo "Failed to read from Apache config file $apache_conf_path \n";
			return $lines;
		}

		$rawData = new Block('serv', 'serv', 0, false);
		$startLine = 0;
		$this->parseLines($lines, $startLine, $rawData, $apache_conf_path);

		if ($this->_debug)
			var_dump($rawData);

		return $rawData;
	}

	/**
	 * Load and parse the Apache configurations.
	 *
	 * @param array $lines An array of the configurations, indexed by line number.
	 * @param int $lineIndex The starting index of the configuration $lines.
	 * @param Block $block The parent block to add new configurations to.
	 * @param string $fileName The file name of the configuration.
	 */
	function parseLines(&$lines, &$lineIndex, &$block, $fileName)
	{
		$c = count($lines);
		if ($this->_debug)
			echo " call parseLines $lineIndex c=$c $fileName\n";
		$end_tag = $block->getEndTag();

		while ($lineIndex < $c)
		{
			$line = trim($lines [$lineIndex]);
			$lineIndex ++;

			if (strlen($line) == 0 || $line [0] == '#')
				continue; // bypass comments

			if ($this->_debug)
				echo " curLine $lineIndex\n";

			if (preg_match("/^<([A-z]+)\s+(.*)>$/", $line, $matches))
			{

				$blockType = $matches [1];
				$blockName = $matches [2];
				$supported = true;
				if (strcasecmp($blockType, 'IfDefine') == 0)
					continue;
				if (strcasecmp($blockType, 'IfModule') == 0)
					$supported = self::isModuleSupported($blockName);
				if ($this->_debug)
					echo " start new block (supported= $supported ): $line\n";
				$allowDup = self::isDupBlockAllowed($blockType);
				$child = new Block($blockType, $blockName, $block->_depth + 1, $allowDup);
				$this->parseLines($lines, $lineIndex, $child, $fileName);
				if ($supported)
					$block->addChildBlock($child);
			}
			else
			{
				if ((!is_null($end_tag))
						&& strncmp($end_tag, $line, strlen($end_tag)) == 0)
				{
					if ($this->_debug)
						echo " end $end_tag new block: $line\n";
					return;
				}

				$path = MigrUtil::checkTag('Include ', $line);
				if ($path != NULL)
				{
					if ($this->_debug)
						echo " found Include directive = $path \n";

					$lists = &MigrUtil::getIncludeFiles($this->_apacheServerRoot, $path);
					if ($lists != NULL)
					{
						foreach ($lists as $includeFile => $includeLines)
						{
							$includeStart = 0;
							$this->parseLines($includeLines, $includeStart, $block, $includeFile);
						}
					}
					else
						echo " failed to include $path\n";
				}
				else
				{
					$entry = new Entry($line, $lineIndex, $fileName);
					if (!isset($this->_apacheServerRoot))
					{
						$this->_apacheServerRoot = MigrUtil::checkTag('ServerRoot ', $line);
						if ($this->_apacheServerRoot != NULL)
							$entry->setStatus(MigrUtil::STATUS_PROCESSED, null);
					}
					$block->addAttribute($entry);

					if ($this->_debug)
						echo "  add genAttr $line\n";
				}
			}
		}
	}

	/**
	 * Check if the module is something we need to add to the ZConf message.
	 *
	 * @param string $module The name of the module to check.
	 */
	static function isModuleSupported($module)
	{
		$matches = array();
		if (!preg_match('/^(?:mod_(.+)\.c)|(?:(.+)_module)$/',
				$module, $matches))
			return false;
		$match = (!empty($matches[1]) ? $matches[1] : $matches[2]);
		return in_array($match, self::$SupportedModules);
	}

	/**
	 * Check if multiple entries of the block type are permitted.
	 *
	 * @param string $block Block type to check.
	 */
	static function isDupBlockAllowed($block)
	{
		return in_array($block, self::$AllowDupBlock);
	}
}