<?php

/********************************************
* LiteSpeed AP2ZConf project
* @Author:   LiteSpeed Technologies, Inc. (http://www.litespeedtech.com)
* @Copyright: (c) 2019
*********************************************/

# Utility functions.

class MigrUtil
{

	const STATUS_UNKNOWN = 0;
	const STATUS_PROCESSED = 1;
	const STATUS_IGNORED = 2;
	const STATUS_ERROR = 3;
	const STATUS_NOT_SUPPORTED = 4;

	/**
	 * Get the $tag value if $line begins with $tag.
	 *
	 * @param string $tag The tag to check
	 * @param string $line The line containing the tag.
	 */
	static function checkTag($tag, $line)
	{
		if (strncasecmp($line, $tag, strlen($tag)) == 0)
		{
			return trim(substr($line, strlen($tag)), " \"\'");
		}
		else
			return NULL;
	}

	static function getBool($val)
	{
		return ((strncasecmp($val, 'On', 2) == 0) ? 1 : 0);
	}

	static function getCommaList($val)
	{
		$arr = preg_split("/[\s]+/", $val);
		if ($arr === false)
			return '';
		return implode(',', $arr);
	}

	static function splitToList($val)
	{
		return preg_split("/[\s\'\"]+/", $val);
	}

	/**
	 * Build an absolute path given the base path and the relative path.
	 *
	 * @param string $root The base path for configurations, if needed.
	 * @param string $path The specified configuration path.
	 * @return string|NULL The combined path.
	 */
	static function getAbsPath($root, $path)
	{
		if ($path[0] != '/')
		{
			if (!isset($root))
				return NULL;
			$absPath = $root . '/' . $path;
			$absPath = str_replace('//', '/', $absPath);
			return $absPath;
		}
		else
			return $path;
	}

	/**
	 * Compile a list of files to include.
	 *
	 * @param string $root The base path for configurations.
	 * @param string $includePath The path to include.
	 * @return array|NULL NULL on failure, String array of files on success.
	 */
	static function &getIncludeFiles($root, $includePath)
	{
		$lists = NULL;
		if (strlen($includePath) == 0)
			return $lists;

		$includePath = MigrUtil::getAbsPath($root, $includePath);
		if ($includePath == NULL)
			return $lists;

		$files = array();
		if (is_file($includePath))
		{
			$files [] = $includePath;
		}
		else
		{
			$pattern = '';
			$pos = strpos($includePath, '*');
			if ($pos === FALSE)
				$pos = strpos($includePath, '?');
			if ($pos !== FALSE)
			{
				$pos1 = strrpos($includePath, '/');
				if ($pos1 === FALSE || $pos1 > $pos)
					return $lists;
				$pattern = substr($includePath, $pos1 + 1);
				$pattern = str_replace('?', '.', $pattern);
				$pattern = str_replace('*', '(.*)', $pattern);
				$pattern = '/^' . $pattern . '$/';
				$includePath = substr($includePath, 0, $pos1);
			}

			if (is_dir($includePath))
			{
				if ($handle = opendir($includePath))
				{
					while (false !== ($file = readdir($handle)))
					{
						if ($file != "." && $file != "..")
						{
							if ($pattern != NULL && !preg_match($pattern, $file))
								continue;

							$file = $includePath . '/' . $file;
							echo " include file = $file \n";
							if (is_file($file))
								$files [] = $file;
						}
					}
					closedir($handle);
				}
			}
		}

		if (count($files) == 0)
			return $lists;

		$lists = array();
		foreach ($files as $f)
		{
			$lines = file($f);
			if ($lines != false)
				$lists [$f] = $lines;
		}

		return $lists;
	}

}
