<?php

/********************************************
* LiteSpeed AP2ZConf project
* @Author:   LiteSpeed Technologies, Inc. (http://www.litespeedtech.com)
* @Copyright: (c) 2017
*********************************************/

# VHostConf coalesces the domains that share IP, port, and SSL.

class VHostConf
{
	private $ip;
	private $port;
	private $nameVH;
	private $binding;
	private $secure;
	private $ssl_cert;
	private $ssl_key;
	private $ssl_cabundle;
	private $ssl_cipher;
	private $vhmap;

	public function __construct($ip, $port)
	{
		$this->ip = $ip;
		$this->port = $port;
		$this->nameVH = 0;
		$this->binding = '';
		$this->secure = 0;
		$this->ssl_cert = '';
		$this->ssl_key = '';
		$this->ssl_cabundle = '';
		$this->ssl_cipher = '';
		$this->vhmap = array();
	}

	function getIp()
	{
		return $this->ip;
	}

	function setIp($ip)
	{
		$this->ip = $ip;
	}

	function getPort()
	{
		return $this->port;
	}

	function setPort($port)
	{
		$this->port = $port;
	}

	function isNameVH()
	{
		return $this->nameVH;
	}

	function setIsNameVH()
	{
		$this->nameVH = 1;
	}

	function setBinding($b)
	{
		$this->binding = $b;
	}

	function getBinding()
	{
		return $this->binding;
	}

	function importVHSsl($aSsl)
	{
		$this->secure = 1;
		$this->ssl_cert = $aSsl['sslcertificatefile'];
		if (isset($aSsl['sslcertificatekeyfile']))
			$this->ssl_key = $aSsl['sslcertificatekeyfile'];
		if (isset($aSsl['sslcertificatechainfile']))
			$this->ssl_cabundle = $aSsl['sslcertificatechainfile'];
		if (isset($aSsl['sslciphersuite']))
			$this->ssl_cipher = $aSsl['sslciphersuite'];
	}

	function importLSWSSsl($lswsListenerConf)
	{
		$this->secure = 1;
		if (isset($lswsListenerConf['certFile']))
			$this->ssl_cert = $lswsListenerConf['certFile'];
		if (isset($lswsListenerConf['keyFile']))
			$this->ssl_key = $lswsListenerConf['keyFile'];
		if (isset($lswsListenerConf['CACertFile']))
			$this->ssl_cabundle = $lswsListenerConf['CACertFile'];
		if (isset($lswsListenerConf['ciphers']))
			$this->ssl_cipher = $lswsListenerConf['ciphers'];
	}

	function isSecure()
	{
		return $this->secure;
	}

	function getSslCert()
	{
		return $this->ssl_cert;
	}

	function getSslKey()
	{
		return $this->ssl_key;
	}

	function getSslCaBundle()
	{
		return $this->ssl_cabundle;
	}

	function getSslCipher()
	{
		return $this->ssl_cipher;
	}

	function appendVHMap($name, $domains)
	{
		if (isset($this->vhmap[$name]))
			$this->vhmap[$name] .= ',' . $domains;
		else
			$this->vhmap[$name] = $domains;
	}

	function getVHMap()
	{
		return $this->vhmap;
	}
};
