#!/bin/sh
eval 'if [ -x /usr/local/cpanel/3rdparty/bin/perl ]; then exec /usr/local/cpanel/3rdparty/bin/perl -x -- $0 ${1+"$@"}; else exec /usr/bin/perl -x -- $0 ${1+"$@"};fi'
if 0;
#!/usr/bin/perl

#WHMADDON:ladc_zconf:LiteSpeed ADC ZConf Manager:webadc_zconf.png
#ACLS:all
####################################################
# Copyright 2017 LiteSpeed Technologies
# http://www.litespeedtech.com
####################################################

use strict;
use warnings;
use FindBin;
use lib FindBin::again();

package cgi::webadc_zconf::config;

use zconflib::utils;

use Data::Validate::IP                  qw(is_ip is_ipv6);
use Data::Validate::Domain              qw(is_domain);
use Storable                            qw(store);
use CGI                                 qw(param);

use Cpanel::Template                    qw(process_template);
use Cpanel::Env                         ;
use Cpanel::Validate::Username::Core    qw(reserved_username_check);

my $confPath = zconflib::utils::getPluginConfFile();

my $paramInputPrefix = 'ls';

my $paramAdcList        = 'adclist';
my $paramConfName       = 'confname';
my $paramAuth           = 'auth';
my $paramExcludeUsers   = 'excludeusers';
my $paramClusterIps     = 'clusterips';
my $paramMaxConns       = 'maxconns';

my $inputAdcList        = $paramInputPrefix . $paramAdcList;
my $inputConfName       = $paramInputPrefix . $paramConfName;
my $inputAuth           = $paramInputPrefix . $paramAuth;
my $inputExcludeUsers   = $paramInputPrefix . $paramExcludeUsers;
my $inputClusterIps     = $paramInputPrefix . $paramClusterIps;
my $inputMaxConns       = $paramInputPrefix . $paramMaxConns;

my %curParams;
my @errorMsgs;
my @successMsgs;
my @warningMsgs;
my @infoMsgs;

run() unless caller();

sub _saveConf {
    my $success;
    my $errormsg;
    if (!zconflib::utils::backupFile(
            zconflib::utils::getPluginConfFile,
            \$errormsg,
            'confs',
            '\.conf')
     ) {
        push @errorMsgs, $errormsg;
        return;
    }
    my $status = eval {
        $success = store(\%curParams, $confPath);
        1;
    };
    if (!defined($status)) {
        push @errorMsgs, "Attempt to save configuration died. $!";
    }
    elsif (!defined($success)) {
        push @errorMsgs, "IO error during configuration file storing. Please check the permissions of $confPath";
    }
    else {
        push @successMsgs, "Configurations file updated.";
        return 1;
    }
}

sub _validateAddr {
    my ($entry) = @_;
    my $port;
    if ($entry =~ s/^\[(.*)\]:(\d+)$/$1/) {
        $port = $2;
        if (!is_ipv6($entry)) {
            push @warningMsgs, "Entry matched IPv6 pattern but was not a valid IPv6 address. [$entry]:$port";
            return;
        }
    }
    elsif ($entry =~ s/:(\d+)$//) {
        $port = $1;
        if (!is_ip($entry) && !is_domain($entry)) {
            push @warningMsgs, "Entry was not a valid IP nor a valid domain. $entry:$port";
            return;
        }
    }
    else {
        push @warningMsgs, "Entry did not contain a port. $entry";
        return;
    }
    return 1 if ($port >= 0 && $port <= 65535);
    push @warningMsgs, "Entry contained an invalid port number: $port";
    return;
}

sub _validateIp {
    my ($entry) = @_;
    if (!is_ip($entry) && !is_ipv6($entry)) {
        push @warningMsgs, "Entry was not a valid IP. $entry";
        return;
    }
    return 1;
}

sub _validateExcludeUser {
    my ($user) = @_;
    if ($user =~ /[\W_]/) {
        push @errorMsgs, "User names should be alphanumeric. The following entry is not alphanumeric: $user";
        return;
    }
    elsif (Cpanel::Validate::Username::Core::reserved_username_check($user)) {
        push @errorMsgs, "User names matched reserved username/pattern. Cannot exclude: $user";
        return;
    }
    return 1;
}

#parse the input
sub _parsePost {
    my ($adclist, $confname, $auth, $excludeusers, $clusterips, $maxconns);
    my $q = CGI->new;

    if (!$q->param('webadc_zconf_conf')) {
        push @errorMsgs, 'Hidden input missing, disregard save.';
        return;
    }

    my $webadclist = $q->param($inputAdcList);
    if ($webadclist) {
        $adclist = join(',', grep(_validateAddr($_), split("[ ,]+", $webadclist)));
    }

    $confname = $q->param($inputConfName);

    $auth = $q->param($inputAuth);
    if ($auth && !(-f $auth)) {
        push @warningMsgs, "Auth file does not exist. Please create the file and enter the configuration again. $auth";
        $auth = $curParams{$paramAuth};
    }

    my $lsexcludeusers = $q->param($inputExcludeUsers);
    if ($lsexcludeusers) {
        # Could just use regex here, but want to add error messages for the bad user names.
        $excludeusers = join(',', grep(_validateExcludeUser($_), split(/\s*,\s*/, $lsexcludeusers)));
    }

    my $listclusterips = $q->param($inputClusterIps);
    if ($listclusterips) {
        # Could just use regex here, but want to add error messages for the bad IP.
        $clusterips = join(',', grep(_validateIp($_), split(/\s*,\s*/, $listclusterips)));
    }

    $maxconns = $q->param($inputMaxConns);
    if ($maxconns ne "") {
        if ($maxconns !~ /^\d+\z/) {
            push @errorMsgs, "Input max conns is not a positive integer.";
            return
        } elsif ((1 > $maxconns) || (2000 < $maxconns)) {
            push @errorMsgs, "Input max conns is not in range 1-2000. $maxconns";
            return
        }
    }

    no warnings 'uninitialized';
    if (!($adclist eq $curParams{$paramAdcList})
      || !($confname eq $curParams{$paramConfName})
      || !($auth eq $curParams{$paramAuth})
      || !($excludeusers eq $curParams{$paramExcludeUsers})
      || !($clusterips eq $curParams{$paramClusterIps})
      || !($maxconns eq $curParams{$paramMaxConns})) {
        $curParams{$paramAdcList} = $adclist;
        $curParams{$paramConfName} = $confname;
        $curParams{$paramAuth} = $auth;
        $curParams{$paramExcludeUsers} = $excludeusers;
        $curParams{$paramClusterIps} = $clusterips;
        $curParams{$paramMaxConns} = $maxconns;
        return if (_saveConf());
    }
    push @infoMsgs, "Configurations file unchanged.";
}

sub _loadConf {
    my ($paramref, $errorref);
    if (zconflib::utils::loadPluginConf(\$paramref, \$errorref)) {
        if ($errorref) {
            push @errorMsgs, $errorref;
        }
        else {
            %curParams = %{$paramref};
        }
    }
    $curParams{$paramAdcList}       = '' unless $curParams{$paramAdcList};
    $curParams{$paramConfName}      = '' unless $curParams{$paramConfName};
    $curParams{$paramAuth}          = '' unless $curParams{$paramAuth};
    $curParams{$paramExcludeUsers}  = '' unless $curParams{$paramExcludeUsers};
    $curParams{$paramClusterIps}    = '' unless $curParams{$paramClusterIps};
    $curParams{$paramMaxConns}      = '' unless $curParams{$paramMaxConns};
}

sub _addTemplateParams {

    $curParams{'template_file'} = 'webadc_zconf/config.tmpl';
    $curParams{'print'} = 1;

    my @callouts;
    if (@errorMsgs) {
        push @callouts, {
            type => 'danger',
            msgs => \@errorMsgs
        };
    }

    if (@warningMsgs) {
        push @callouts, {
            type => 'warning',
            msgs => \@warningMsgs
        };
    }

    if (@infoMsgs) {
        push @callouts, {
            type => 'info',
            msgs => \@infoMsgs
        };
    }

    if (@successMsgs) {
        push @callouts, {
            type => 'success',
            msgs => \@successMsgs
        };
    }

    $curParams{'webadc_alerts'} = \@callouts;

    my $security_token = $ENV{'cp_security_token'} || '';
    my @configEntries = (
        {
            inTitle => 'Server Conf Name',
            inName => $inputConfName,
            inId => $paramConfName,
            inValue => $curParams{$paramConfName},
            required => 1,
            descTitle => 'What is Server Conf Name?',
            descSyntax => 'Alphanumeric characters are recommended for ease of use.',
            descValue => <<END
The Server Conf Name is the conf name used by the ADC to identify this backend server.
This should be unique to this server.
If another backend server uses the same conf name, the old conf will be overridden.
END
        },
        {
            inTitle => 'ADC Basic Auth',
            inName => $inputAuth,
            inId => $paramAuth,
            inValue => $curParams{$paramAuth},
            required => 1,
            descTitle => 'What is ADC Basic Auth?',
            descSyntax => 'This should be the absolute path to the credentials file.',
            descValue => <<END
ADC Basic Auth is the full path to a file containing the user:password to use.
The content is expected to be in user:password format, with only ONE ':'.
If multiple entries exist in the file, only the first one with the correct format will be used.
END
        },
        {
            inTitle => 'ADC List',
            inName => $inputAdcList,
            inId => $paramAdcList,
            inValue => $curParams{$paramAdcList},
            required => 1,
            descTitle => 'What is ADC List?',
            descSyntax => 'Spaces and commas delimit the list of ADCs and each ADC should be of IP:port or domain:port format.',
            descValue => <<END
The ADC List is the list of ZConf Configured ADCs to which the ZConf messages will be sent.
Each listed ADC should already be configured to listen for ZConf messages at the port given.
END
        },
        {
            inTitle => 'Exclude Users',
            inName => $inputExcludeUsers,
            inId => $paramExcludeUsers,
            inValue => $curParams{$paramExcludeUsers},
            descTitle => 'What is Exclude Users?',
            descSyntax => 'Spaces and commas delimit the list of users and each user must be alphanumeric and may not start with \'test\'.',
            descValue => <<END
This is a list of users that should be excluded from forwarding to the configured ADCs.
Example reasons to exclude a user include: User transferred to another server,
User has a custom set up, etc.
END
        },
        {
            inTitle => 'Cluster IPs',
            inName => $inputClusterIps,
            inId => $paramClusterIps,
            inValue => $curParams{$paramClusterIps},
            descTitle => 'What is Cluster IPs?',
            descSyntax => 'Spaces and commas delimit the list of IPs. Append attributes with semi-colon.',
            descValue => <<END
This is a list of additional backend IPs to configure for all* domains on this server. If a cluster configuration is specified for the
domain using Userdata, that will override this list for that domain.
Example usage: If this server is replicated across multiple servers without its own WHM WebADC ZConf manager.
Read more about cluster attributes in the <a href="$security_token/cgi/webadc_zconf/per_domain_customizations.cgi" >Per Domain Customizations page.</a>
END
        },
        {
            inTitle => 'Max Conns',
            inName => $inputMaxConns,
            inId => $paramMaxConns,
            inValue => $curParams{$paramMaxConns},
            descTitle => 'What is Max Conns?',
            descSyntax => 'Integer value between 1 and 2000 inclusive.',
            descValue => <<END
This is the maximum number of concurrent connections that each ADC worker process will utilize to visit each backend. For example,
if there are two ADCs each with two worker processes, there will be at most 4 * maxConns concurrent connections to this backend. Note: if there are
Cluster IPs configured, they will default to use the same max conn value unless specified otherwise.
END
        }
    );
    $curParams{'webadc_configentries'} = \@configEntries;
}

sub run {
    print "Content-type: text/html; charset=utf-8\n\n";

    if (zconflib::utils::validateRoot(0)) {
        _loadConf();

        if ($ENV{'REQUEST_METHOD'} eq "POST") {
            _parsePost();
        }
        _addTemplateParams();

        Cpanel::Template::process_template(
            'whostmgr',
            \%curParams
        );
    }
    exit();
}

