#!/bin/sh
eval 'if [ -x /usr/local/cpanel/3rdparty/bin/perl ]; then exec /usr/local/cpanel/3rdparty/bin/perl -x -- $0 ${1+"$@"}; else exec /usr/bin/perl -x -- $0 ${1+"$@"};fi'
if 0;
#!/usr/bin/perl

#WHMADDON:ladc_zconf:LiteSpeed ADC ZConf Manager:webadc_zconf.png
#ACLS:all
####################################################
# Copyright 2017 LiteSpeed Technologies
# http://www.litespeedtech.com
####################################################

use strict;
use warnings;
use Fcntl ':flock';
use FindBin;
use lib FindBin::again();

package cgi::webadc_zconf::generate_zconf;

use zconflib::utils;
require "send_server_conf.cgi";

use Cpanel::Config::Httpd           qw(httpd_vendor_info);
use Cpanel::ConfigFiles::Apache     qw(apache_paths_facade);
use Cpanel::Binaries                ();
use Cpanel::Config::userdata::Load  qw(get_all_domains_for_user load_userdata user_exists);
use Cpanel::Config::userdata::Utils qw(get_all_vhost_domains_from_vhost_userdata);
use Cpanel::Config::WebVhosts       qw(load ssl_proxy_subdomains_for_vhost);

use Whostmgr::Accounts              qw(listsuspended);

my %OPTS = @ARGV;

exit() unless (zconflib::utils::validateRoot(1, \%OPTS));

my $failed = 1;
my $script = $FindBin::Bin . '/generatezconf.php';
my $msg;

# Information extracted from Cpanel's API. Set in _getValidCpanelInfo()
my ($phpBin, $apacheConf, $lswsHome);

my @excludedDomains;

my $pluginConfRef;

if (!open my $self, '<', $0) {
    $msg = "ERROR: failed to open generate.";
}
elsif (!flock($self, Fcntl::LOCK_EX | Fcntl::LOCK_NB)) {
    $msg = ''; # No message needed for locked file.
}
elsif (zconflib::utils::openLog()) {
    if (zconflib::utils::loadPluginConf(\$pluginConfRef, \$msg)
        && !$msg
        && _getValidCpanelInfo()) {
        @excludedDomains = _getExcludedDomains($pluginConfRef->{'excludeusers'});
        _genZconf();
        if (%OPTS) {
            zconflib::utils::log("Try to update the servers.\n");
            cgi::webadc_zconf::send_server_conf::run();
        }
    }
    zconflib::utils::closeLog();
}
else {
    $msg = "ERROR: failed to open log for writing: $!";
}

if (%OPTS) {
    print STDERR $msg if ($msg);
}
else {
    unless ($msg) {
        $msg = 'ERROR: No message set.';
    }
    zconflib::utils::sendJson(!$failed, $msg);
}


exit();

sub _getValidCpanelInfo {

    my @serverSoftware = Cpanel::Config::Httpd::httpd_vendor_info();
    if ($serverSoftware[0] =~ /^litespeed$/) { # NOTE: ap2zconf does not accept OLS configs.
        $lswsHome = '/usr/local/lsws';
    }
    elsif (!($serverSoftware[0] =~ /apache/)) {
        $msg = 'ERROR: The server software selected is not supported by this plugin yet.';
        return;
    }
    # else it is an Apache setup.

    if (defined(&Cpanel::Binaries::get_binary_location)) {
        $phpBin = Cpanel::Binaries::get_binary_location('php');
    }
    elsif (defined(&Cpanel::Binaries::path)) {
        $phpBin = Cpanel::Binaries::path('php');
    }
    else {
        $msg = 'ERROR: No known path search function available, please submit a ticket to LiteSpeed for a fix.';
        return;
    }
    unless (-x $phpBin) {
        $msg = 'ERROR: Retrieved php binary is not executable.';
        return;
    }

    $apacheConf = apache_paths_facade->file_conf;
    unless (-f $apacheConf) {
        $msg = 'ERROR: Retrieved Apache config file path is not a file.';
        return;
    }

    return 1;
}

sub _getExcludedDomains {
    my ($confExcluded) = @_;
    my %domainList; # Domain list is a hash to ensure a list of unique domains.
    my @excludeUserNames;
    my @userServerNames;
    my @createdDomains;
    my @proxySubdomains;
    my $userData;
    my $curVhost;
    my $suspendList = Whostmgr::Accounts::listsuspended();

    @excludeUserNames = split(/,/, $confExcluded) if $confExcluded;

    # Check for accounts transferred using express transfer.
    foreach my $suspendedAcctRef (@$suspendList) {
        my $printuser = ${$suspendedAcctRef}{'user'};
        zconflib::utils::log("suspended user $printuser\n");

        if ((${$suspendedAcctRef}{'reason'})
                && (${$suspendedAcctRef}{'reason'} eq "User transferred to another server.")) {
            zconflib::utils::log("USER WAS TRANSFERRED $printuser\n");
            push @excludeUserNames, ${$suspendedAcctRef}{'user'};
        }
        else {
            zconflib::utils::log("USER WAS OTHER $printuser\n");
        }
    }

    return unless @excludeUserNames;

    foreach my $suspendedUser (@excludeUserNames) {
        next if (!Cpanel::Config::userdata::Load::user_exists($suspendedUser));

        @userServerNames = Cpanel::Config::userdata::Load::get_all_domains_for_user($suspendedUser);
        $curVhost = Cpanel::Config::WebVhosts->load($suspendedUser);
        foreach my $serverName (@userServerNames) {
            $userData = Cpanel::Config::userdata::Load::load_userdata($suspendedUser, $serverName, 0);
            @createdDomains = Cpanel::Config::userdata::Utils::get_all_vhost_domains_from_vhost_userdata($userData);
            @proxySubdomains = $curVhost->ssl_proxy_subdomains_for_vhost($serverName);

            @domainList{@createdDomains} = 1;
            @domainList{@proxySubdomains} = 1;
        }
    }
    #TODO: Should I sort?
    return (%domainList ? (sort { reverse($a) cmp reverse($b) } keys %domainList) : ());
}

sub _genZconf {
    my $tpDir = zconflib::utils::getThirdPartyDir();

    zconflib::utils::log("BEGIN GENERATE\n");
    foreach my $excl (@excludedDomains) {
        zconflib::utils::log("exclude $excl\n");
    }
    my @assoc_args = (
                    '--apacheconf=' . $apacheConf,
                    '--zconfdir=' . $tpDir,
                    '--lswshome=' . $lswsHome
    );

    if ($pluginConfRef->{'clusterips'}) {
            push @assoc_args, '--clusterips=' . $pluginConfRef->{'clusterips'};
    }

    if ($pluginConfRef->{'maxconns'}) {
            push @assoc_args, '--maxconns=' . $pluginConfRef->{'maxconns'};
    }

    my $pid = open(my $phpRes, "-|", $phpBin, '-f', $script, '--',
                    @assoc_args,
                    '--', @excludedDomains); # Excluded domains are numbered parameters.
    if (defined($pid)) {
        while (my $line = <$phpRes>) {
            zconflib::utils::log($line);
        }
        my $phpRet = close($phpRes);
        if ($phpRet) {
            $failed = 0;
            $msg = "Successfully generated ZConf Message Files in $tpDir";
        }
        elsif ($!) {
            $msg = "ERROR: A system call failed or something bad happened. $!";
        }
        else {
            $msg = "ERROR: Generating ZConf Messages produced an error. $!";
        }
    }
    else {
        $msg = "ERROR: Failed to initiate ZConf Message Generation: $!";
    }
    zconflib::utils::log("END GENERATE\n");
}

