#!/bin/sh
eval 'if [ -x /usr/local/cpanel/3rdparty/bin/perl ]; then exec /usr/local/cpanel/3rdparty/bin/perl -x -- $0 ${1+"$@"}; else exec /usr/bin/perl -x -- $0 ${1+"$@"};fi'
if 0;
#!/usr/bin/perl

#WHMADDON:ladc_zconf:LiteSpeed ADC ZConf Manager:webadc_zconf.png
#ACLS:all
####################################################
# Copyright 2017 LiteSpeed Technologies
# http://www.litespeedtech.com
####################################################

use strict;
use warnings;
use FindBin;
use lib FindBin::again();

package cgi::webadc_zconf::send_server_conf;

use zconflib::utils;

use CGI                                 qw(param);
use LWP::Protocol::https;
use LWP::UserAgent          qw(post);
use Path::Tiny              qw(path);
use Encode                  qw(encode);

use Cpanel::JSON            qw(Dump);

my $confFile = zconflib::utils::getPluginConfFile();
my $serverUpFile = zconflib::utils::getServerUpFile();
my $sslForceUpdateFile = zconflib::utils::getSslForceUpdateFile();
my $sslMsgFile = zconflib::utils::getSslMsgFile();
my $sslReleaseMsgFile = zconflib::utils::getSslReleaseMsgFile();
my $errorMsg;
my $msg;

my @adcList;
my $auth;
my $serverId;

my %OPTS = @ARGV;

run() unless caller();

sub run {

    exit() unless (zconflib::utils::validateRoot(1, \%OPTS));

    if (!(-f $confFile)) {
        $errorMsg = 'ADC Configuration is not set up yet.';
    }
    elsif (!(-f $serverUpFile) || !(-f $sslMsgFile)) {
        $errorMsg = 'Server conf files not generated yet.';
    }
    elsif (_loadConf()) {
        #conf loaded ok.
        my $q = CGI->new;
        my $forceSsl = $q->param('forcessl');
        foreach my $adc (@adcList) {
            if ((-f $sslReleaseMsgFile) && !_sendMsg($adc, 'ZCSSLRELEASE', $sslReleaseMsgFile)) {
                next;
            }
            if ($forceSsl) {
                if ((-f $sslForceUpdateFile) && !_sendMsg($adc, 'ZCSSLRELEASE', $sslForceUpdateFile)) {
                    next;
                }
                else {
                    $msg .= "Successfully force removed SSL from $adc<br>";
                }
            }
            if (!_sendMsg($adc, 'ZCUP', $serverUpFile)) {
                next;
            }
            elsif (!_sendMsg($adc, 'ZCSSL', $sslMsgFile)) {
                next;
            }
            $msg .= "Successfully sent configs to $adc<br>";
        }
    }

    unless ($errorMsg && !-f $sslReleaseMsgFile) {
        unlink $sslReleaseMsgFile;
    }

    if (!%OPTS) {
        if ($errorMsg) {
            $msg .= $errorMsg;
        }

        zconflib::utils::sendJson($errorMsg ? 0 : 1, $msg);
    }
    exit();
}

sub _parseConf {
    my $fh;
    my ($storedConfRef) = @_;

    $serverId = encode("UTF-8", $storedConfRef->{'confname'});
    @adcList = split(',', $storedConfRef->{'adclist'});

    my $filename = $storedConfRef->{'auth'};
    unless (open($fh, '<', $filename)) {
        $errorMsg = "Could not open file '$filename' $!";
        return;
    }

    while (my $line = <$fh>) {
        # The below check does not account for commented lines.
        # If more time should be spent on this plugin, will need to fix this.
        if ($line =~ '^[^:]+:[^:]+$') { # Look for a user:password line
            $line =~ s/\015?\012?$//; # Remove crlf from all OS groups.
            $auth = $line;
            last;
        }
    }

    unless (close($fh)) {
        $errorMsg = "Could not close file '$filename' $!";
        return;
    }

    unless ($auth) {
        $errorMsg = 'No valid auth string found';
        return;
    }
    return 1;
}

sub _loadConf {
    my $storedConfRef;
    return unless ((zconflib::utils::loadPluginConf(\$storedConfRef, \$errorMsg))
                    && (defined($storedConfRef)));

    unless (($storedConfRef->{'adclist'}) && ($storedConfRef->{'confname'}) && ($storedConfRef->{'auth'})) {
        $errorMsg = 'Required configurations are not set.';
        return;
    }

    return _parseConf($storedConfRef);
}

sub _sendMsg {
    my ($domain, $cmd, $file) = @_;
    my $url = "https://$domain/$cmd?name=$serverId";
    my $ua = LWP::UserAgent->new();
    my ($user, $pass) = split(/:/, $auth);
    $ua->ssl_opts(verify_hostname => 0 ,SSL_verify_mode => 0x00);
    $ua->credentials($domain, 'ZeroConfig Restricted Area', $user, $pass);
    my $filepath = path($file);
    my $filecontents = $filepath->slurp;
    my $res = $ua->post($url,
        Content => $filecontents
    );
    my $code = $res->code();
    if ($code == 200) {
        return 1;
    }
    elsif ($code == 500) {
        $errorMsg .= <<END
Failed to send command $cmd to $domain. This could be because the ADC is not configured to listen on that port.
This could also be an issue related to server firewall settings, if you or your provider customized them. 
To test, please make sure that a basic curl command from the server to the ADC works.
If the curl test returns that it couldn't connect to host, there is an issue.<br>
END
    }
    elsif ($code == 401) {
        $errorMsg .= <<END
Failed to send command $cmd to $domain, auth parameters did not work. Please check the passwd files for 
content correctness and location accessibility.<br>
END
    }
    else {
        $errorMsg .= "Sending command $cmd to $domain resulted in error $code<br>";
    }
    return;
}

