#!/bin/bash

# /********************************************
# WEBADC ZConf Manager Plugin for WHM
# @Author:   LiteSpeed Technologies, Inc. (http://www.litespeedtech.com)
# @Copyright: (c) 2017
# *********************************************/

cd $(dirname "${0}") > /dev/null
CURDIR=$(pwd -L)
cd - > /dev/null

UPGRADING=0
CPANELDIR="/usr/local/cpanel"
DOCROOT="${CPANELDIR}/whostmgr/docroot"
INSDIR="${DOCROOT}/cgi/webadc_zconf"
LIBDIR="${INSDIR}/zconflib"
TMPLDIR="${DOCROOT}/templates/webadc_zconf"
ICONDIR="${DOCROOT}/addon_plugins"
TRDPTYDIR="${CPANELDIR}/3rdparty/webadc_zconf"
SCRIPTSDIR="${CPANELDIR}/scripts"
CRONFILE="/etc/cron.d/whm_webadc_zconf_detect_update"

if [ ! -d "$CPANELDIR" ] ; then
    exit
fi

CUR_USER=`id`
CUR_USER=`expr "$CUR_USER" : 'uid=.*(\(.*\)) gid=.*'`

if [ $CUR_USER != "root" ]  ; then
    echo "Require root permission to install this plugin. Abort!"
    exit
fi

function is_less_than()
{
    ver1="${1}"
    ver2="${2}"
    isOlderThan=$(perl -e "print ${ver1} < ${ver2};")
}

echo ""
echo "========================================================="
echo " Install WEBADC ZConf Manager Plugin for WHM"
echo ""

# Check for and create the directory for plugin and AppConfig files.
if [ ! -d /var/cpanel/apps ]
  then
    mkdir /var/cpanel/apps
    chmod 755 /var/cpanel/apps
    echo "  Created cPanel apps directory"
fi

# Check for and create the directory for plugin CGI files.
if [ ! -d "$INSDIR" ]
  then
    mkdir "$INSDIR"
    chmod 755 "$INSDIR"
    echo "  Created plugin CGI directory $INSDIR"
fi

# Check for and create the directory for plugin lib files.
if [ ! -d "$LIBDIR" ]
  then
    mkdir "$LIBDIR"
    chmod 755 "$LIBDIR"
    echo "  Created plugin lib directory $LIBDIR"
fi

# Check for and create the directory for plugin template files.
if [ ! -d "$TMPLDIR" ]
  then
    mkdir "$TMPLDIR"
    chmod 755 "$TMPLDIR"
    echo "  Created plugin template directory $TMPLDIR"
fi

# Check for and create the directory for plugin thirdparty files.
if [ ! -d "$TRDPTYDIR" ]
  then
    mkdir -p "$TRDPTYDIR/confs/"
    chmod 755 "$TRDPTYDIR"
    echo "  Created plugin third party directory $TRDPTYDIR"
else
    OLDVER=$(cat "$TRDPTYDIR/VERSION")
fi

CURVER=$(cat "$CURDIR/VERSION")

is_less_than "${OLDVER}" "0.7"
if [ "$isOlderThan" = "1" ]; then
    # If older than 0.7, need to re-register app to update VHost Template page to Per Domain Customizations.
    /usr/local/cpanel/bin/register_appconfig "$CURDIR/webadc_zconf.conf"
fi

if [ -e "/usr/local/cpanel/bin/register_appconfig" ] ; then
    REGISTERED=`/usr/local/cpanel/bin/is_registered_with_appconfig whostmgr webadc_zconf`
fi

if [ $REGISTERED -eq 1 ] ; then
    UPGRADING=1
else
    # Register the plugin with AppConfig.
    echo "  Registering the plugin in WHM"
    /usr/local/cpanel/bin/register_appconfig "$CURDIR/webadc_zconf.conf"
fi

# Copy plugin files to their locations and update permissions.
echo "  Copy files into CGI directory $INSDIR"
/bin/cp -f "$CURDIR/install.sh" "$INSDIR/"
/bin/cp -f "$CURDIR/uninstall.sh" "$INSDIR/"
/bin/cp -f "$CURDIR/cgi/webadc_zconf/"* "$INSDIR/"
/bin/cp -f "$CURDIR/cgi/webadc_zconf/zconflib/"* "$LIBDIR/"
chmod -R 755 "$INSDIR/"
/bin/cp -f "$CURDIR/webadc_zconf.conf" "$INSDIR/"
/bin/cp -f "$CURDIR/LICENSE" "$INSDIR/"
/bin/cp -R -f "$CURDIR/cgi/webadc_zconf/static" "$INSDIR/"
/bin/cp -R -f "$CURDIR/ap2zconf" "$INSDIR/"

echo "  Copy files into templates directory $TMPLDIR"
/bin/cp -R -f "$CURDIR/templates/webadc_zconf/"* "$TMPLDIR/"
/bin/cp -f "$CURDIR/cgi/webadc_zconf/static/icons/webadc_zconf.png" "$ICONDIR/"

echo "  Copy files into third party directory $TRDPTYDIR"
/bin/cp -f "$CURDIR/VERSION" "$TRDPTYDIR/"

# echo "  Create script entries for automated serverup"
## List of scripts to hook to. REMOVED "postpkgacct", maybe duplicate with manage hooks below?
# declare -a scripthooks=("postmodifyacct" "postrestoreacct" "postsuspendacct" "postunsuspendacct" "postupdateuserdomains")
# declare -a scripthooks=("postsuspendacct" "postunsuspendacct")

# for scripthook in "${scripthooks[@]}"
# do
#     if [ ! -f "$SCRIPTSDIR/$scripthook" ]
#     then
#         echo "#!/bin/bash" > "$SCRIPTSDIR/$scripthook"
#         chmod 755 "$SCRIPTSDIR/$scripthook"
#     fi
#     echo "$INSDIR/postactionupdate.sh & # webadc_zconf auto update" >> "$SCRIPTSDIR/$scripthook"
# done

if [ $UPGRADING -eq 1 ]; then
    echo "  UPGRADING, remove existing hooks and cron tasks."
    echo "  Remove Standardized hooks."
    /usr/local/cpanel/bin/manage_hooks delete script $INSDIR/stdscript

    echo "...removing cron task..."
    if [ -f "$CRONFILE" ]; then
        rm -f "$CRONFILE"
    fi
fi

echo "  Add Standardized Hooks"
/usr/local/cpanel/bin/manage_hooks add script "$INSDIR/stdscript"

echo "Add hourly change detection cron to $CRONFILE"
echo "31      *       *       *       *       root    $INSDIR/detect_update" > "$CRONFILE"
chmod 600 "$CRONFILE"

echo ""
echo " WEBADC ZConf Manager WHM Plugin Installed Successfully."
echo "========================================================="


